"""sysinfo web user interface"""

from cubicweb.web import uicfg
from cubicweb.web.views.urlrewrite import SimpleReqRewriter


class SysinfoSimpleReqRewriter(SimpleReqRewriter):
    """handle main entry points::

    /services : list of services

    /inventory : inventory

    /applications : application list
    """
    rules = [
        ('/services', dict(vid='services',rql='Service S')),
        ('/inventory', dict(vid='devices',rql='Device D')),
        ('/applications', dict(vid='applications',rql='Application D')),
        ]

uicfg.autoform_section.tag_subject_of(('Application', 'has_instance', '*'),
                                        'main', 'attributes')
uicfg.autoform_section.tag_subject_of(('Service', 'has_entry_point', '*'),
                                        'main', 'attributes')
uicfg.autoform_section.tag_subject_of(('Devide', 'made_of', '*'),
                                        'main', 'attributes')



"""sysinfo specific startup views

:organization: Logilab
:copyright: 2006-2009 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
__docformat__ = "restructuredtext en"


from cubicweb.web.views import startup

class IndexView(startup.ManageView):
    __regid__ = 'index'

    def call(self):
        self.display_actions()
        self.display_services()


    def display_actions(self):
        _ = self._cw._
        w = self.w
        w(u'<table class="xxx"><tr>')
        w(u'<td><a href="/services">%s</a></td>' % _(u"Services"))
        w(u'<td><a href="/inventory">%s</a></td>' % _(u"Inventaire"))
        w(u'</tr></table>')

    def display_services(self):
        rql = "Any S, H, P, D WHERE S is Service, S purpose P, S description D, S hosted_by H?"
        rset = self._cw.execute(rql)
        self.wview('table', rset, 'null')

def registration_callback(vreg):
    vreg.register(IndexView, clear=True)



from logilab.common.testlib import unittest_main
from cubicweb.devtools.testlib import AutomaticWebTest

class AutomaticWebTest(AutomaticWebTest):
    no_auto_populate = ('Repository', 'Revision', 'VersionedFile',
                        'VersionContent', 'DeletedVersionContent')

    def to_test_etypes(self):
        return set(('Host','Application', 'NetworkService', 'Service'))

    def list_startup_views(self):
        return ('index',)

if __name__ == '__main__':
    unittest_main()



"""jpl application'schema

:organization: Logilab
:copyright: 2003-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
__docformat__ = "restructuredtext en"

from yams.buildobjs import (EntityType, RelationDefinition, SubjectRelation,
                            String, Date, Int)

try:
    from cubes.file.schema import File, Image
    from cubes.link.schema import Link
except (ImportError, NameError):
    # old-style yams schema will raise NameError on EntityType, RelationType, etc.
    File = import_erschema('File')
    Image = import_erschema('Image')
    Link = import_erschema('Link')

_ = unicode

PERMISSIONS = {'read':   ('managers', 'staff', 'users', 'guests',),
               'add':    ('managers', 'staff', ),
               'update': ('managers', 'staff', 'owners',),
               'delete': ('managers', 'staff', 'owners'),
               }

File.__permissions__ = PERMISSIONS
Image.__permissions__ = PERMISSIONS
Link.__permissions__ = PERMISSIONS

class Part(EntityType):
    name     = String(required=True, fulltextindexed=True, unique=False,
                      maxsize=64, description=_('Name, short description of the part')
                      )

    serial  = String(required=False, fulltextindexed=True, unique=False,
                     maxsize=128, description=_('Part serial number')
                     )

    description = String(fulltextindexed=True,
                         description=_('Detailed description of the part'))

#     part_type = ObjectRelation('PartType', cardinality='11')
    concerned_by = SubjectRelation('File', cardinality='?*')

    situated_in = SubjectRelation('Zone', cardinality='?*')

    warranty_expires = Date(description=_('End of waranty'))


class Device(EntityType):
    name     = String(required=True, fulltextindexed=True, unique=False,
                      maxsize=64, description=_('Name, short description of the device')
                      )

    serial  = String(required=False, fulltextindexed=True, unique=False,
                     maxsize=128, description=_('Device serial number')
                     )

    klass = String(vocabulary=('server',
                               'desktop',
                               'thinclient',
                               'laptop',
                               'display',
                               'printer',
                               'switch',
                               'other',
                               ),
                   description=_('Indicates the general class of this device'))


    description = String(fulltextindexed=True,
                         description=_('Detailed description of the device'))

    made_of = SubjectRelation('Part', cardinality='1*')

    concerned_by = SubjectRelation('File', cardinality='**')

    situated_in = SubjectRelation('Zone', cardinality='?*')

    warranty_expires = Date(description=_('End of waranty'))

# class PartType(EntityType):
#     name     = String(required=True, fulltextindexed=True, unique=False,
#                       maxsize=64,
#                       description=_('Name, short description of the device'))


# the follwing relations still need to be described in this file
# Comment comments File

class Host(EntityType):
    fqdn = String(required=True, fulltextindexed=True, unique=True,
                  maxsize=255,
                  description=_("Fully qualified domain name of this host"))

    ip_addr = String(required=True, fulltextindexed=True, unique=True,
                     maxsize=255, description=_("IP address"))

    hosted_by = SubjectRelation( ('Device', 'Host'), cardinality='1*' )

    os = String(required=True)

class NetworkService(EntityType):
    """Network service as in /etc/services"""
    name = String(required=True, description=_('Service name'))
    port = Int(required=True, description=_('Service port'))
    type = String(required=True, vocabulary=('tcp','udp'), description=_('port type'))

class Application(EntityType):
    """Description of a network application"""
    title = String(required=True,
                   maxsize=255,
                   description=_('Application title'), unique=True)
    description = String(description=_("Application description"))

    has_instance = SubjectRelation( 'Service', cardinality='*1', composite='subject')

class Service(EntityType):
    """An entity describing a service of an IT infrastructure"""

    name = String(required=True, fulltextindexed=True, unique=True,
                  description=_("Service identifier"),
                  maxsize=255,
                  )


    description = String(required=True, fulltextindexed=True,
                         description=_("Detailled description"))


    purpose = String(required=True, vocabulary=('prod', 'dev', 'test', 'deprecated'),
                     description=_('Indicates the intended purpose of this service'))





class comments(RelationDefinition):
    subject = 'Comment'
    object = ('File', 'VersionedFile', 'Service', 'Device', 'Part')


class tags(RelationDefinition):
    subject = 'Tag'
    object = ('File',
              'Image',
              'Link',
              'VersionedFile')

class has_entrypoint(RelationDefinition):
    subject = 'Service'
    object = ('Link',)

class hosted_by(RelationDefinition):
    subject = 'Service'
    object = ('Host',)


class admin_doc(RelationDefinition):
    subject = ('VersionedFile',)
    object = ('Application', 'Service')
    cardinality = '*?'

class user_doc(RelationDefinition):
    subject = ('VersionedFile',
               'Link'
               )
    object = ('Application', 'Service')
    cardinality = '*?'



# -*- coding: utf-8 -*-

from cubicweb.entities import AnyEntity

class Application(AnyEntity):
    __regid__ = 'Application'


class Service(AnyEntity):
    __regid__ = 'Service'


class Device(AnyEntity):
    __regid__ = 'Device'


class Host(AnyEntity):
    __regid__ = 'Host'




#add_cube('vcsfile')
#add_cube('link')
#add_cube('tag')

add_entity_type('Host')
add_entity_type('NetworkService')
add_entity_type('Application')
add_entity_type('Service')

synchronize_rschema('comments')
add_cubes('tags', 'link', 'vcsfile')

#add_relation_definition('Service', 'has_entrypoint', 'Link')
#
#add_relation_definition('Service', 'hosted_by', 'Host')
#
#add_relation_definition('VersionedFile', 'admin_doc', 'Application')
#add_relation_definition('VersionedFile', 'admin_doc', 'Service')
#
#add_relation_definition('VersionedFile', 'user_doc', 'Application')
#add_relation_definition('VersionedFile', 'user_doc', 'Service')
#add_relation_definition('Link', 'user_doc', 'Application')
#add_relation_definition('Link', 'user_doc', 'Service')
#
add_attribute('Part', 'warranty_expires')
add_attribute('Device', 'warranty_expires')




from cubicweb.selectors import implements
from cubicweb.web.views import autoform

class HostForm(autoform.AutomaticEntityForm):
    __select__ = implements('Host')

    def subject_hosted_by_vocabulary(self, rtype, limit=None):
        hosts = self.req.execute('Host H')
        devices = self.req.execute('Device D')
        if hosts:
            lst = [ (u'Hosts', None) ] + [ (e.view('combobox'), e.eid)
                                           for e in hosts.entities() ]
        if device:
            lst+= [ (u'Devices', None)]+ [ (e.view('combobox'), e.eid)
                                           for e in devices.entities() ]
        return lst



# pylint: disable-msg=W0622
"""cubicweb-sysinfo packaging information"""

modname = 'sysinfo'
distname = "cubicweb-sysinfo"

numversion = (0, 11, 0)
version = '.'.join(str(num) for num in numversion)

license = 'LGPL'
copyright = '''Copyright (c) 2003-2010 LOGILAB S.A. (Paris, FRANCE).
http://www.logilab.fr/ -- mailto:contact@logilab.fr'''

author = "Logilab"
author_email = "contact@logilab.fr"

short_desc = "sysinfo component for the CubicWeb framework"
long_desc = """This CubicWeb component provides a computer inventory system
(to manage an inventory of computers, printers, keyboards, displays, etc).

CubicWeb is a semantic web application framework, see http://www.cubicweb.org
"""

web = 'http://www.cubicweb.org/project/%s' % distname

classifiers = [
    'Environment :: Web Environment',
    'Framework :: CubicWeb',
    'Programming Language :: Python',
    'Programming Language :: JavaScript',
    ]

# dependencies

__depends_cubes__ = {'zone': None,
                     'file': None,
                     'folder': None,
                     'comment': None,
                     'link': None,
                     'tag': None,
                     'vcsfile': None,
                     }
__depends__ = {'cubicweb': '>= 3.6.0'}
for key, value in __depends_cubes__.items():
    __depends__['cubicweb-'+key] = value
__use__ = tuple(__depends_cubes__)

# packaging

from os import listdir
from os.path import join, isdir
import glob

pyversions = ['2.4']

CUBES_DIR = join('share', 'cubicweb', 'cubes')
try:
    data_files = [
        # server / client python files
        [join(CUBES_DIR, 'sysinfo'),
         [fname for fname in listdir('.')
          if fname.endswith('.py') and fname != 'setup.py']],
        # modpython client files
        [join(CUBES_DIR, 'sysinfo', 'data'),
         [join('data', fname) for fname in listdir('data') if fname != 'CVS']],
        [join(CUBES_DIR, 'sysinfo', 'i18n'),
         [join('i18n', fname) for fname in listdir('i18n')
          if fname != 'CVS' and not isdir(join('i18n', fname))]],
        [join(CUBES_DIR, 'sysinfo', 'views'),
         [join('views', fname) for fname in listdir('views')
          if fname != 'CVS' and not fname.endswith('.pyc')]],
        # server files
        [join(CUBES_DIR, 'sysinfo', 'migration'),
         [join('migration', fname) for fname in listdir('migration')
          if fname != 'CVS']],
        ]
except OSError:
    # we are in an installed directory
    pass
